#ifdef HAVE_CONFIG_H
# include "config.h"     
#endif
#include <iostream>
#include"dune/common/mpihelper.hh" // An initializer of MPI
#include"dune/common/exceptions.hh" // We use exceptions

#include<math.h>
#include<vector>
#include<map>
#include<string>

#include<dune/common/fvector.hh>
#include<dune/common/static_assert.hh>
#include<dune/common/timer.hh>
#include<dune/grid/io/file/vtk/subsamplingvtkwriter.hh>
#include<dune/grid/io/file/gmshreader.hh>
#include<dune/grid/yaspgrid.hh>

#include<dune/istl/bvector.hh>
#include<dune/istl/operators.hh>
#include<dune/istl/solvers.hh>
#include<dune/istl/preconditioners.hh>
#include<dune/istl/io.hh>
#include<dune/istl/superlu.hh>

#include<dune/pdelab/common/function.hh>
#include<dune/pdelab/common/vtkexport.hh>

#include<dune/pdelab/finiteelementmap/q22dfem.hh>
#include<dune/pdelab/finiteelementmap/q1fem.hh>
#include<dune/pdelab/finiteelementmap/conformingconstraints.hh>
#include<dune/pdelab/instationary/onestep.hh>

#include<dune/pdelab/gridfunctionspace/gridfunctionspace.hh>
#include<dune/pdelab/gridfunctionspace/gridfunctionspaceutilities.hh>
#include<dune/pdelab/gridfunctionspace/genericdatahandle.hh>
#include<dune/pdelab/gridfunctionspace/interpolate.hh>
#include<dune/pdelab/gridfunctionspace/constraints.hh>
#include<dune/pdelab/gridoperatorspace/gridoperatorspace.hh>
#include<dune/pdelab/gridoperatorspace/instationarygridoperatorspace.hh>
#include<dune/pdelab/backend/istlvectorbackend.hh>
#include<dune/pdelab/backend/istlmatrixbackend.hh>
#include<dune/pdelab/backend/istlsolverbackend.hh>
#include<dune/pdelab/stationary/linearproblem.hh>

#include"learn01_bctype.hh"
#include"learn01_bcextension.hh"
#include"learn01_operator.hh"
#include"learn01_Q1.hh"
//#include"learn01_Q2.hh"

int main(int argc, char** argv)
{
  try{
    //Maybe initialize Mpi
    Dune::MPIHelper& helper = Dune::MPIHelper::instance(argc, argv);
    std::cout << "Hello World! This is test-par." << std::endl;
    if(Dune::MPIHelper::isFake)
      std::cout<< "This is not a sequential program!" << std::endl;
    else
    {
		if(helper.rank()==0)
		  std::cout << "parallel run on " << helper.size() << " process(es)" << std::endl;
    }

     // std::cout<<"I am rank "<<helper.rank()<<" of "<<helper.size()
     //   <<" processes!"<<std::endl;
     if (argc!=2)
     {
	if(helper.rank()==0)
	  std::cout << "usage: ./example01 <level>" << std::endl;
	return 1;
    }
	
    int level;
    sscanf(argv[1],"%d",&level);
	
      // make grid
      Dune::FieldVector<double,2> L(2.0);
      Dune::FieldVector<int,2> N(1);
      Dune::FieldVector<bool,2> B(false);
      int overlap=3;
      Dune::YaspGrid<2> grid(helper.getCommunicator(),L,N,B,overlap);
      grid.globalRefine(level);
      typedef Dune::YaspGrid<2>::LeafGridView GV;
      const GV& gv=grid.leafView();

      /*Dune::FieldVector<double,2> L(1.0);
      Dune::FieldVector<int,2> N(16);
      Dune::FieldVector<bool,2> periodic(false);
      int overlap=0;
      //Dune::YaspGrid<2> grid(L,N,periodic,overlap);
      typedef Dune::YaspGrid<2> grid(helper.getCommunicator(), L, N, periodic, overlap);
      grid.globalRefine(level);
      typedef Dune::YaspGrid<2>::LeafGridView GV;
      const GV& gv=grid.leafView();*/
      std::cout << std::endl << "Initialization Complete." << std::endl;
      std::cout << std::endl << "Now performing learn01_Q1(gv)" << std::endl;
      learn01_Q1(gv);
      std::cout << "Completed." << std::endl;
      //std::cout << std::endl << "Now performing learn01_Q2(gv)" << std::endl;
      //learn01_Q2(gv);
      //std::cout << "Completed." << std::endl;
    //}
	
	
    return 0;
  }
  catch (Dune::Exception &e){
    std::cerr << "Dune reported error: " << e << std::endl;
  }
  catch (...){
    std::cerr << "Unknown exception thrown!" << std::endl;
  }
}
